import { Component } from 'vue';
import { ComponentOptionsMixin } from 'vue';
import { ComponentProvideOptions } from 'vue';
import { CSSProperties } from 'vue';
import { DefineComponent } from 'vue';
import { ExtractPropTypes } from 'vue';
import { InjectionKey } from 'vue';
import { PropType } from 'vue';
import { PublicProps } from 'vue';
import { RendererElement } from 'vue';
import { RendererNode } from 'vue';
import { VNode } from 'vue';

export declare type ComponentType = Component | DefineComponent;

export declare interface Field {
    name: string;
    props: Props;
    slots?: Recordable<() => void>;
    on?: Recordable<string | Function | undefined>;
}

export declare type InjectionType = {
    readonly?: boolean;
    [key: string]: any;
};

export declare interface Props {
    key?: string | number;
    style?: CSSProperties;
    class?: string[];
    [key: string]: any;
}

export declare type Recordable<T = any> = Record<string, T>;

export declare const RENDER_INJECTION_KEY: InjectionKey<string>;

export declare class RenderElement<T extends Field = Field> {
    hooks: Recordable<(field: T) => void>;
    globalHook?: (field: T) => void;
    components: Recordable<ComponentType>;
    /**
     * constructor function
     */
    constructor(globalHook?: (field: T) => void);
    /**
     * register components
     * @param name
     * @param component
     * @param hook
     */
    addComponent(name: string, component: ComponentType, hook?: (field: T) => void): this;
    /**
     * remove components
     * @param name
     */
    removeComponent(name: string): this;
    /**
     * rendering components
     */
    render(): DefineComponent<ExtractPropTypes<    {
    field: {
    type: PropType<T>;
    required: true;
    };
    modelValue: {
    type: PropType<any>;
    required: false;
    default: undefined;
    };
    }>, () => VNode<RendererNode, RendererElement, {
    [key: string]: any;
    }>, {}, {}, {}, ComponentOptionsMixin, ComponentOptionsMixin, {
        'update:modelValue': (value: any) => true;
        input: (value: any) => true;
    }, string, PublicProps, Readonly<ExtractPropTypes<    {
    field: {
    type: PropType<T>;
    required: true;
    };
    modelValue: {
    type: PropType<any>;
    required: false;
    default: undefined;
    };
    }>> & Readonly<{
        "onUpdate:modelValue"?: ((value: any) => any) | undefined;
        onInput?: ((value: any) => any) | undefined;
    }>, {
        modelValue: any;
    }, {}, {}, {}, string, ComponentProvideOptions, true, {}, any>;
}

export { }
